﻿using System;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Runtime.CompilerServices;

namespace TaskCancellationExample.ViewModels
{
  public abstract class ViewModelBase : INotifyPropertyChanged, IDisposable
  {
    /// <summary>
    /// Event is cast whenever a property changes.
    /// </summary>
    public event PropertyChangedEventHandler PropertyChanged;

    private bool _disposed = false; // To detect redundant calls

    protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
    {
      PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
    }

    protected virtual void OnPropertyChanged<T>(Expression<Func<T>> expression)
    {
      if (expression == null)
      {
        throw new ArgumentNullException(nameof(expression));
      }

      var memberExpression = (MemberExpression)expression.Body;
      OnPropertyChanged(memberExpression.Member.Name);
    }

    /// <summary>
    /// Implementation of IDisposable. Calls Dispose(true).
    /// </summary>
    public void Dispose()
    {
      Dispose(true);
      GC.SuppressFinalize(this);
    }

    // The bulk of the clean-up code is implemented in Dispose(bool)
    protected virtual void Dispose(bool disposing)
    {
      if (_disposed)
      {
        return;
      }

      if (disposing)
      {
        // free managed resources
      }

      // free native resources if there are any.
      _disposed = true;
    }
  }
}